/*************************************************************
 File:		FTDI_IO.c
 Project:	USB PTP\MTP Camera Controller

 Author:					Luke Skaff
 Author's Email:		Luke.Skaff@gmail.com
 Author's Website:	http://blog.LukeSkaff.com/
							http://www.LukeSkaff.com/
 ************************************************************/

#include "FTDI_IO.h"


/* FTDI Command values */
volatile unsigned char out_IPH[4]={0x49,0x50,0x48,0x0D};
volatile unsigned char out_SCS[4]={0x53,0x43,0x53,0x0D};
volatile unsigned char out_DSD[4]={0x83,0x20,0x00,0x0D};

volatile unsigned char debugIOenable=FALSE_B;
volatile unsigned char debugIOenableASCII=FALSE_B;

struct USB_RX_Type {
	/* 0xFF = read finished, 0x00 = read unfinished */
	unsigned char USBpipeReadDone;
	unsigned char USBsessionOpen;

	/* Total Size of data to be read from USB device */
	uint32_t totalSize;
	/* Size of data chunk FTDI will return, if totalSize is larger then the
	   USB pipe out end point size */
	unsigned char chunkSize;

	/* Keeps track of which byte is being read */
	uint32_t currentTotalByteIndex;
	unsigned char currentChunkByteIndex;
};

struct USB_RX_Type USB_RX=
{
	.USBpipeReadDone=TRUE_B,
	.USBsessionOpen=FALSE_B,
	.totalSize=0x00000000,
	.chunkSize=0x00,
	.currentTotalByteIndex=0x00000000,
	.currentChunkByteIndex=0x00
};

/******************************************************************
*******************************************************************
*******************************************************************
Start of platform specific functions
*******************************************************************
*******************************************************************
*******************************************************************/

/******************************************************************
 * Function: FTDI_TX_Byte_NoWait
 * Send byte to FTDI
 ******************************************************************
*/
unsigned char FTDI_TX_Byte_NoWait(unsigned char outByte)
{

	/* AVR specific code */
	#ifdef USE_PLATFORM_AVR
		/* If FTDI_TXE_ is low */
		if(bit_is_clear(FTDI_CTRL_PORT_IN, FTDI_TXE_))
		{
		
		
			bit_Set(FTDI_CTRL_PORT,FTDI_WR);
			FTDI_IO_CTRL=0xFF; /* Change FTDI port to output */
			FTDI_DATA_PORT=outByte;

			__asm__ volatile("nop");
			__asm__ volatile("nop");

			bit_Clear(FTDI_CTRL_PORT,FTDI_WR); /* Clear WR pin */

			__asm__ volatile("nop");
			__asm__ volatile("nop");

			FTDI_DATA_PORT=0x00; /* Disable any pullups */
			FTDI_IO_CTRL=0x00; /* Change FTDI port to input */
		#endif


		/* Nintendo DS specific code */
		#ifdef USE_PLATFORM_NDS
			/* If FTDI_TXE_ is low */
			if(bit_is_clear(FTDI_CTRL_REG, FTDI_TXE_))
			{

			bit_Set(FTDI_CTRL_REG,(FTDI_WR|FTDI_WR_EN)); /* Enable WR pin & WR_EN pin */
			FTDI_DATA_REG=outByte; /* Put data on FTDI data lines */

			/* Add delay if needed */
			bit_Clear(FTDI_CTRL_REG,FTDI_WR); /* Clear WR pin */
			/* Add delay if needed */
	
			bit_Clear(FTDI_CTRL_REG,FTDI_WR_EN); /* Turn off output driver on CPLD */
		#endif


		/**** Debug Output ****/
		#ifdef DEBUG_IO_OUTPUT
			Debug_Out(outByte);
		#endif
		#ifdef DEBUG_IO_OUTPUT_SELECTIVE
			if(debugIOenable == TRUE_B)
				Debug_Out(outByte);
		#endif
		#ifdef DEBUG_IO_OUTPUT_SELECTIVE_ASCII
			if(debugIOenableASCII == TRUE_B)
				Debug_Print_HEX_to_ASCII(outByte);
		#endif
	
		return ECODE_NO_ERROR;
	}

	/* If FTDI_TXE_ is NOT low, FTDI not ready for new data, return time out error */
	return ECODE_FTDI_TX_TIMEDOUT;
}

/******************************************************************
 * Function: FTDI_RX_Byte_NoWait
 * Send byte to FTDI
 ******************************************************************
*/
unsigned char FTDI_RX_Byte_NoWait(volatile unsigned char *inByte)
{

	#ifdef USE_PLATFORM_AVR
		if(bit_is_clear(FTDI_CTRL_PORT_IN, FTDI_RXF_))
		{
			/* Start read */
			bit_Clear(FTDI_CTRL_PORT,FTDI_RD_);

			__asm__ volatile("nop");
			__asm__ volatile("nop");
			__asm__ volatile("nop");
	
			*inByte=FTDI_DATA_PORT_in;
	
			bit_Set(FTDI_CTRL_PORT,FTDI_RD_); /* End read */
	#endif


	#ifdef USE_PLATFORM_NDS
		if(bit_is_clear(FTDI_CTRL_REG, FTDI_RXF_))
		{
			/* Start read */
			bit_Set(FTDI_CTRL_REG,FTDI_RD);

		   /* Add wait if needed */
	
			*inByte=FTDI_DATA_REG;
	
			bit_Clear(FTDI_CTRL_REG,FTDI_RD); /* End read */
	#endif



		/* Debug Only */
		#ifdef DEBUG_IO_OUTPUT
			Debug_Out(*inByte);
		#endif
		#ifdef DEBUG_IO_OUTPUT_SELECTIVE
			if(debugIOenable == TRUE_B)
				Debug_Out(*inByte);
		#endif
		#ifdef DEBUG_IO_OUTPUT_SELECTIVE_ASCII
			if(debugIOenableASCII == TRUE_B)
				Debug_Print_HEX_to_ASCII(*inByte);
		#endif

		return ECODE_NO_ERROR;
	}

	return ECODE_FTDI_RX_TIMEDOUT;
}

/******************************************************************
*******************************************************************
*******************************************************************
 End of platform specific functions
*******************************************************************
*******************************************************************
*******************************************************************/





/*
******************************************************************
* Function: FTDI_TX_Byte
* Send byte to FTDI
******************************************************************
*/
unsigned char FTDI_TX_Byte(unsigned char outByte, unsigned int timeout)
{
	unsigned char errorCode=ECODE_NO_ERROR;
	volatile unsigned int count;
	
	/* Wait up to timeout for new data before timeout */
	for(count=0 ; count<timeout ; count++)
	{
		errorCode=FTDI_TX_Byte_NoWait(outByte);
		if (errorCode == ECODE_NO_ERROR)		
			return ECODE_NO_ERROR;
		else
			delay_1ms();
	}

	return ECODE_FTDI_TX_TIMEDOUT;
}



/*
******************************************************************
* Function: FTDI_RX_Byte
* Send byte to FTDI
******************************************************************
Error Codes:
0x00 = No Error;
0x01 = Timeout,
*/
unsigned char FTDI_RX_Byte(volatile unsigned char *inByte, unsigned int timeout)
{
	unsigned char errorCode=ECODE_NO_ERROR;
	volatile unsigned int count;


	/* Wait up to timeout (ms) to receive byte */
	for(count=0 ; count<timeout ; count++)
	{

		errorCode=FTDI_RX_Byte_NoWait(inByte);

		if(errorCode == ECODE_NO_ERROR)
			return ECODE_NO_ERROR;
		else
			delay_1ms();

	}

	return ECODE_FTDI_RX_TIMEDOUT;
}



/*
***************************************************************************
 FTDI_TX_4B
***************************************************************************
*/
unsigned char FTDI_TX_4B(volatile unsigned char outData[4], unsigned char TXsize,unsigned int timeout)
{
	volatile unsigned char i;
	unsigned char tmp;
	unsigned char errorCode=ECODE_NO_ERROR;

	for(i=0 ; i < TXsize ; i++)
	{
		tmp=FTDI_TX_Byte(outData[i],timeout);
		
		if (tmp != 0x00)
			errorCode=tmp;
	}

	return errorCode;
}



/*
***************************************************************************
 FTDI_Dummy_RD_Timeout
***************************************************************************
*/
unsigned char FTDI_Dummy_RD_Timeout(unsigned int timeout_ms)
{
	unsigned char tmp;
	unsigned char status=0x0F;
	unsigned char dummyByte;
	unsigned char firstByte=0xFF;
	unsigned int i=0;
	unsigned int count=0;

	for (i = 0; i < timeout_ms; i++)
	{
		/* AVR specific code */
		#ifdef USE_PLATFORM_AVR
			if(bit_is_clear(FTDI_CTRL_PORT_IN, FTDI_RXF_))
		#endif

		/* Nintendo DS specific code */
		#ifdef USE_PLATFORM_NDS
			if(bit_is_clear(FTDI_CTRL_REG, FTDI_RXF_))
		#endif
		{
			/* If this is the first byte being read clear the status code */
			if(firstByte==0xFF)
			{
				status=0x00;
				firstByte=0x00;
			}

			/* If read count exceeds limit */
			if(count > 300)
				return ECODE_FTDI_READ_MAXED;


			tmp=FTDI_RX_Byte_NoWait(&dummyByte);
			if (tmp != 0x00)
				status=tmp;

			i=0; //reset timeout loop
			count++;
		}
		else
			delay_1ms();
	}
	return status;
}


/*
******************************************************************
 Function: FTDI_INT                                             
 Enable and Initialize FTDI                                     
******************************************************************
*/
unsigned char FTDI_init(void)
{
	unsigned char notDone;
	unsigned char errorCode;
	unsigned char inByte;
	unsigned char lastByte;

	/* Toggle Reset pin on FTDI */

	/* AVR specific code */
	#ifdef USE_PLATFORM_AVR
		bit_Clear(FTDI_CTRL_PORT,FTDI_RESET_);
		delay_ms(TIMEOUT_DEFAULT);
		bit_Set(FTDI_CTRL_PORT,FTDI_RESET_);
	#endif

	/* Nintendo DS specific code */
	#ifdef USE_PLATFORM_NDS
		bit_Clear(FTDI_CTRL_REG,FTDI_RESET_);
		delay_ms(TIMEOUT_DEFAULT);
		bit_Set(FTDI_CTRL_REG,FTDI_RESET_);
	#endif


	/* Look for ':' symbol (0x3A), then end of line 0x0D */
	notDone=TRUE_B;
	lastByte=FALSE_B;
	while(notDone == TRUE_B)
	{
		errorCode=FTDI_RX_Byte(&inByte, TIMEOUT_LONG2x);

		/* If last two bytes were 0x3A and 0x0D exit while loop */
		if( (lastByte == 0x3A) && (inByte == 0x0D) )
			notDone=0x00;

		lastByte=inByte;

		/* If error exit */
		if(errorCode != ECODE_NO_ERROR)
		{
			/* Debug */
			#ifdef DEBUGoutput
			Debug_Print('1', errorCode);
			#endif

			return errorCode; /* Return error code */
		}

	}

	/* Dump "Device Detected Px" if USB device connected
	   if USB device not connected this will just timeout */
	FTDI_Dummy_RD_Timeout(TIMEOUT_LONG);



	/* Set FTDI IPH mode, 4 bytes lone, 16ms wait */
	errorCode=FTDI_TX_4B(out_IPH,0x04, TIMEOUT_DEFAULT);
	if (errorCode != ECODE_NO_ERROR)
	{
		#ifdef DEBUGoutput
		Debug_Print('2', errorCode);
		#endif
		return errorCode;
	}

	errorCode=FTDI_Dummy_RD_Timeout(TIMEOUT_DEFAULT);
	if (errorCode != ECODE_NO_ERROR)
	{
		#ifdef DEBUGoutput
		Debug_Print('3', errorCode);
		#endif
		return errorCode;
	}


	/* Set SCS mode, 4 bytes lone, 16ms wait */
	errorCode=FTDI_TX_4B(out_SCS, 0x04, TIMEOUT_DEFAULT);
	if (errorCode != ECODE_NO_ERROR)
	{
		#ifdef DEBUGoutput
		Debug_Print('4', errorCode);
		#endif
		return errorCode;
	}


	errorCode=FTDI_Dummy_RD_Timeout(TIMEOUT_DEFAULT);
	if (errorCode != ECODE_NO_ERROR)
	{
		#ifdef DEBUGoutput
		Debug_Print('5', errorCode);
		#endif
		return errorCode;
	}
	
	return ECODE_NO_ERROR;
}


/*
******************************************************************
 Function: FTDI_DRD
 Send DRD command to FTDI then return size of data
******************************************************************
 chunkInSize can only be a maximum size of the USB device pipe out end point size 
*/
unsigned char FTDI_DRD(volatile unsigned char *chunkInSize)
{
	unsigned char errorCode;
	unsigned char tmpInByte1;
	unsigned char tmpInByte2;
	volatile unsigned char i;

	/* Init to 0 for case where read fails */
	*chunkInSize = 0x00;

	/**** Send DRD (Device Read Data) command to FTDI ****
	If a CF (command failed, 43 46 0D) is responded that means USB is not ready (NAK) response
	try again after USB NAK timeout specified in define statement 
	*/
	unsigned char dataGood=FALSE_B;

	for(i=0x00 ; ((i<USB_NAK_RETRY_CNT) && (dataGood == FALSE_B)) ; i++)
	{
		/* Send 0x84 to FTDI */
		errorCode=FTDI_TX_Byte(0x84,TIMEOUT_DEFAULT);
		if (errorCode != 0x00)
		{
			#ifdef DEBUGoutput
				Debug_Detailed(1,8,'1',errorCode);
			#endif
			return errorCode;
		}

		/* Send 0x0D to FTDI */
		errorCode=FTDI_TX_Byte(0x0D,TIMEOUT_DEFAULT);
		if (errorCode != 0x00)
		{
			#ifdef DEBUGoutput
				Debug_Detailed(1,8,'2',errorCode);
			#endif
			return errorCode;
		}

		/* Read size of data packet, first byte */
		errorCode=FTDI_RX_Byte(&tmpInByte1, TIMEOUT_DEFAULT);
		if(errorCode != 0x00)
		{
			//Debug
			#ifdef DEBUGoutput
				Debug_Detailed(1,8,'3',errorCode);
			#endif
			return errorCode; // Return error code
		}

		/* Read back the 0x0D */
		errorCode=FTDI_RX_Byte(&tmpInByte2, TIMEOUT_DEFAULT);
		if(errorCode != 0x00)
		{
			//Debug
			#ifdef DEBUGoutput
				Debug_Detailed(1,8,'4',errorCode);
			#endif
			return errorCode; // Return error code
		}

		/* If return size is not 0x00 and second byte is 0D, data good */
		if( (tmpInByte1 != 0x00) && (tmpInByte2 == 0x0D) )
		{
			dataGood=TRUE_B;
		}
		else /* Dump CF (Command Failed) FTDI sends when error and wait */
		{
			FTDI_Dummy_RD_Timeout(TIMEOUT_DEFAULT);
			delay_ms(DELAY_USB_NAK);
		}
		
		/* If max retry count is reached and data is not good return USB timeout */
		if( (i == (USB_NAK_RETRY_CNT-1)) && (dataGood != 0xFF) )
		{
			#ifdef DEBUGoutput
				Debug_Detailed(1,8,'5',0x00);
			#endif
			return ECODE_FTDI_USB_TIMEDOUT; /* Return USB timeout */
		}
	}

	*chunkInSize=tmpInByte1;
	return ECODE_NO_ERROR;
}


/*
******************************************************************
* Function: FTDI_3E0D_Check
* Check to make sure FTDI returns 0x3E0D
******************************************************************
 When specific commands have been successful, the FTDI vinculum will return 0x3E0D,
 this function checks for this response
*/
unsigned char FTDI_3E0D_Check(void)
{
	unsigned char errorCode;
	unsigned char inByte1;
	unsigned char inByte2;

	/*** Check for FTDI Confirmation Response ***/
	errorCode=FTDI_RX_Byte(&inByte1, TIMEOUT_LONG);
	if (errorCode != ECODE_NO_ERROR)
	{
		#ifdef DEBUGoutput
			Debug_Detailed(1,9,'1',errorCode);
		#endif
		return errorCode;
	}
	
	errorCode=FTDI_RX_Byte(&inByte2, TIMEOUT_DEFAULT);
	if (errorCode != ECODE_NO_ERROR)
	{
		#ifdef DEBUGoutput
			Debug_Detailed(1,9,'2',errorCode);
		#endif
		return errorCode;
	}

	/** Make sure 0x3E, 0x0D was received from FTDI confirming data sent **/
	if ( (inByte1 != 0x3E) || (inByte2 != 0x0D) )
	{
		#ifdef DEBUGoutput
			Debug_Detailed(1,9,'3',errorCode);
		#endif
		return ECODE_FTDI_NO_3E0D;
	}

	return ECODE_NO_ERROR;
}



/*
*******************************************************************************
* Function: USB_RX_Byte
* Reads a byte from USB device
*******************************************************************************
 This function handles all the communication with the FTDI such as 
 sending a DRD command, checking the size, keeping track which data byte has been read,
 how much data is left to read, and more.

 This function will only read a USB data packet where the size of the USB data is read is specified
 in the first 32-bits
*/
unsigned char USB_RX_Byte(volatile unsigned char *inByte)
{
	unsigned char errorCode;


	/*** STEP 1 ***/
	/* if USB read is not in progress, return error code */
	if(USB_RX.USBsessionOpen == FALSE_B)
	{
		#ifdef DEBUGoutput
			Debug_Detailed(1,10,'1',0x00);
		#endif

		return ECODE_USB_RX_BYTE_SESSION_CLOSED;
	}


	/*** STEP 2 ***/
	/*	If USB read is in progress (Session started\open) AND USB pipe is done (fully read)
		send DRD command to FTDI and get new data chunk size to start new read */
	if( (USB_RX.USBsessionOpen == TRUE_B) && (USB_RX.USBpipeReadDone == TRUE_B) )
	{
		/* Send out DRD (Device Read Data) command to FTDI and get the data size */
		errorCode=FTDI_DRD(&USB_RX.chunkSize);
		if(errorCode != ECODE_NO_ERROR)
		{
			/* Used for Debug Only */
			#ifdef DEBUGoutput
				Debug_Detailed(1,10,'2',errorCode);
			#endif

/*
Debug_Out_HEX_to_ASCII_W_CR(USB_RX.chunkSize);
Debug_Out_HEX_to_ASCII_W_CR(USB_RX.currentChunkByteIndex);

Debug_Out_HEX_to_ASCII(USB_RX.totalSize>>24);
Debug_Out_HEX_to_ASCII(USB_RX.totalSize>>16);
Debug_Out_HEX_to_ASCII(USB_RX.totalSize>>8);
Debug_Out_HEX_to_ASCII_W_CR(USB_RX.totalSize);

Debug_Out_HEX_to_ASCII(USB_RX.currentTotalByteIndex>>24);
Debug_Out_HEX_to_ASCII(USB_RX.currentTotalByteIndex>>16);
Debug_Out_HEX_to_ASCII(USB_RX.currentTotalByteIndex>>8);
Debug_Out_HEX_to_ASCII_W_CR(USB_RX.currentTotalByteIndex);
*/

			/* if error, such as no new data to read from USB, default values */
			*inByte=0x00;
			USB_RX_Byte_Reset();

			return errorCode; /* Return error code */
		}

		/* Read in progress & pipe read now in progress */
		USB_RX.USBsessionOpen = 0xFF;
		USB_RX.USBpipeReadDone = FALSE_B;
	}


	/*** STEP 3 ***/

	/* Debug output to terminal */
	#ifdef DEBUG_IO_OUTPUT_USB_RX_ASCII
		debugIOenableASCII=TRUE_B;
	#endif

	/* If reading the first four bytes of a new USB packet load USB RX total size */
	if (USB_RX.currentTotalByteIndex < 0x00000004)
	{
		errorCode=FTDI_RX_Byte(inByte, TIMEOUT_DEFAULT);
		DEBUG_DETAILED_OUTPUT_W_IF(1,10,'3',errorcode);

		/* Shifts each byte of the total size into the correct position in the double word total size var */
		USB_RX.totalSize |= ( ((uint32_t)*inByte)<<(0x00000008 * USB_RX.currentTotalByteIndex) );
	}
	else if (USB_RX.currentTotalByteIndex >= 0x00000004)
	{
		errorCode=FTDI_RX_Byte(inByte, TIMEOUT_DEFAULT);
		DEBUG_DETAILED_OUTPUT_W_IF(1,10,'4',errorcode);
	}

	/* Debug */
	#ifdef DEBUG_IO_OUTPUT_USB_RX_ASCII
		debugIOenableASCII=FALSE_B;
	#endif	

	if(errorCode != ECODE_NO_ERROR)
		{
			/* if error set size to zero and reset function */
			*inByte=0x00;
			USB_RX_Byte_Reset();

			return errorCode;
		}


	/*** STEP 4 ***/
	/* If length (32-bits) has been read AND the last byte of the total USB data reset function and close session for new read */
	if ((USB_RX.currentTotalByteIndex >= 0x00000004) && (USB_RX.currentTotalByteIndex==(USB_RX.totalSize-1)))
	{
		USB_RX_Byte_Reset();

		/** Read back 0x3E0D from FTDI **/
		errorCode=FTDI_3E0D_Check();
		if (errorCode != ECODE_NO_ERROR)
		{
			#ifdef DEBUGoutput
				Debug_Detailed(1,10,'5',errorCode);
			#endif
			return errorCode;
		}
	}
	/* else if reading the last byte of USB chunk\pipe */
	else if(USB_RX.currentChunkByteIndex==(USB_RX.chunkSize-1))
	{
		USB_RX.USBpipeReadDone=TRUE_B;
		USB_RX.currentChunkByteIndex=0x00;
		USB_RX.currentTotalByteIndex++;

		/** Read back 0x3E0D from FTDI **/
		errorCode=FTDI_3E0D_Check();
		if (errorCode != ECODE_NO_ERROR)
		{
			#ifdef DEBUGoutput
				Debug_Detailed(1,10,'6',errorCode);
			#endif

			/* if error set size to zero and reset function */
			*inByte=0x00;
			USB_RX_Byte_Reset();

			return errorCode;
		}
	}
	else
	{
		USB_RX.currentChunkByteIndex++;
		USB_RX.currentTotalByteIndex++;
	}

	return ECODE_NO_ERROR;
}


/*
*******************************************************************************
 Function: USB_RX_Byte_Reset
 Resets all variables of USB_RX global struct
*******************************************************************************
*/
void USB_RX_Byte_Reset(void)
{
	USB_RX.chunkSize = 0x00;
	USB_RX.totalSize = 0x00000000;
	USB_RX.currentChunkByteIndex = 0x00;
	USB_RX.currentTotalByteIndex = 0x00000000;
	USB_RX.USBsessionOpen = FALSE_B;
	USB_RX.USBpipeReadDone = TRUE_B;
}


/*
*******************************************************************************
 Function: USB_RX_Byte_StartSession
 Starts session and returns size of data left to be read from USB device
*******************************************************************************
*/
unsigned char USB_RX_Byte_StartSession(volatile uint32_t *remainingSize)
{
	volatile unsigned char dummyVar;
	unsigned char errorCode;
	unsigned char i;

	/* in case of error default size to zero */
	*remainingSize=0x00000000;

	/*** STEP 1 ***/
	/* If USB read is in progress, return error code */
	if(USB_RX.USBsessionOpen == TRUE_B)
		return ECODE_USB_RX_BYTE_SESSION_IN_PROGRESS;


	/*** STEP 2 ***/
	/* Set session */
	USB_RX.USBsessionOpen=TRUE_B;

	/* Load up the totalSize variable */
	for(i=0 ; i<4 ; i++)
	{
		errorCode=USB_RX_Byte(&dummyVar);

		if(errorCode != ECODE_NO_ERROR)
			return errorCode;
	}


	/*** STEP 3 ***/
	/* load remainingSize Var, since four bytes has been read from the USB device the remaining size
		is subtracted by four */
	*remainingSize = (USB_RX.totalSize - 0x00000004);
	

	return ECODE_NO_ERROR;
}
