/*************************************************************
 File:		PTP_basic.c
 Project:	USB PTP\MTP Camera Controller

 Author:					Luke Skaff
 Author's Email:		Luke.Skaff@gmail.com
 Author's Website:	http://blog.LukeSkaff.com/
							http://www.LukeSkaff.com/
 ************************************************************/


#include "PTP_basic.h"

/*** Global Variables ***/
unsigned char debugWaitForEventAllIO = FALSE_B;

/*** PTP/MTP Canon EOS shutter speed table ***/
/* Each stop is an increment of 0x08 */
const unsigned char PROGMEM Canon_Shutterspeed_Table[] =
{ 
	0x00, /* auto */
	0x04, /* bulb */
	0x0c, /* bulb */
	0x10, /* 30  */
	0x13, /* 25  */
	0x14, /* 20  */ /* + 1/3 */
	0x15, /* 20  */
	0x18, /* 15  */
	0x1b, /* 13  */
	0x1c, /* 10  */
	0x1d, /* 10  */ /* 10.4 */
	0x20, /* 8   */
	0x23, /* 6   */ /* + 1/3 */
	0x24, /* 6   */
	0x25, /* 5   */
	0x28, /* 4   */
	0x2b, /* 3.2 */
	0x2c, /* 3   */
	0x2d, /* 2.5 */
	0x30, /* 2   */
	0x33, /* 1.6 */
	0x34, /* 1.5 */
	0x35, /* 1.3 */
	0x38, /* 1   */
	0x3b, /* 0.8 */
	0x3c, /* 0.7 */
	0x3d, /* 0.6 */
	0x40, /* 0.5 */
	0x43, /* 0.4 */
	0x44, /* 0.3 */
	0x45, /* 0.3 */ /* 1/3 */
	0x48, /* 1/4 */
	0x4b, /* 1/5 */
	0x4c, /* 1/6 */
	0x4d, /* 1/6 */ /* 1/3? */
	0x50, /* 1/8 */
	0x53, /* 1/10 */ /* 1/3? */
	0x54, /* 1/10 */
	0x55, /* 1/13 */
	0x58, /* 1/15 */
	0x5b, /* 1/20 */ /* 1/3? */
	0x5c, /* 1/20 */
	0x5d, /* 1/25 */
	0x60, /* 1/30 */
	0x63, /* 1/40 */
	0x64, /* 1/45 */
	0x65, /* 1/50 */
	0x68, /* 1/60 */
	0x6b, /* 1/80 */
	0x6c, /* 1/90 */
	0x6d, /* 1/100 */
	0x70, /* 1/125 */
	0x73, /* 1/160 */
	0x74, /* 1/180 */
	0x75, /* 1/200 */
	0x78, /* 1/250 */
	0x7b, /* 1/320 */
	0x7c, /* 1/350 */
	0x7d, /* 1/400 */
	0x80, /* 1/500 */
	0x83, /* 1/640 */
	0x84, /* 1/750 */
	0x85, /* 1/800 */
	0x88, /* 1/1000 */
	0x8b, /* 1/1250 */
	0x8c, /* 1/1500 */
	0x8d, /* 1/1600 */
	0x90, /* 1/2000 */
	0x93, /* 1/2500 */
	0x94, /* 1/3000 */
	0x95, /* 1/3200 */
	0x98, /* 1/4000 */
	0x9b, /* 1/5000 */
	0x9c, /* 1/6000 */
	0x9d, /* 1/6400 */
	0xa0, /* 1/8000 */
};

/* USB variables */
unsigned char tmpPTPcontainer[24]={0x00};


struct PTPcontainer_Type {
	unsigned char length;
	uint16_t type;
	uint16_t code;
	unsigned char tID;
	unsigned char sessionID;
	unsigned char numOfParm;
	unsigned char param1[4];
	unsigned char param2[4];
	unsigned char param3[4];
};


struct PTPcontainer_Type PTPcontainerTX=
{
	.sessionID=0x01,
	.tID=0x00
};



/*
Standard PTP Data Packet
Important: Each field is sent LSB first
-------------------------
      | Packet |        |
Byte  | Byte   | Field  |
Offset| Number | Name   | 
-------------------------
  0   |   0    | Length | 
      |   1    |        | 
      |   2    |        |
      |   3    |        |
-------------------------
  4   |   4    | Type   |
      |   5    |        |
-------------------------
  6   |   6    | Code   |
      |   7    |        |
-------------------------
  8   |   8    | tID    |
      |   9    |        |
      |   10   |        |
      |   11   |        |
-------------------------
  12  |   12   | Param1 |
      |   13   |        |
      |   14   |        |
      |   15   |        |
-------------------------
  16  |   16   | Param2 |
      |   17   |        |
      |   18   |        |
      |   19   |        |
-------------------------
  20  |   20   | Param3 |
      |   21   |        |
      |   22   |        |
      |   23   |        |
*/




/******************************************************************
 * Function: PTP_Container_TX                                     *
 * TX PTP Container to out FTDI to USB device                     *
 ******************************************************************
Error codes:
0x00 = No Error
0x01 = Timeout while reading writing to FTDI, reset recommended
0x10 = Timeout while waiting for FTDI to confirm it received USB data packet, reset recommended
*/
unsigned char PTP_Container_TX(void)
{
	unsigned char errorCode;
	unsigned char outSize;
	volatile unsigned char i;
	volatile unsigned char p1;
	volatile unsigned char p2;
	volatile unsigned char p3;

	/* Clear anything in the FTDI output FIFO */
	FTDI_Dummy_RD_Timeout(TIMEOUT_DEFAULT); 

	/* Calculate output packet size */
	outSize=(0x0C + (0x04 * PTPcontainerTX.numOfParm));
	
	/* Send DSD (device send data) command to FTDI */
	/* 0x83,0x20,size,0x0D */
	out_DSD[2]=outSize;
	errorCode=FTDI_TX_4B(out_DSD,0x04,TIMEOUT_DEFAULT);
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Detailed(2,1,'1',errorCode);
		#endif
		return errorCode;
	}


	/**** Send out PTP Packet, each field is sent LSB first ****/
	/* init param counters */
	p1=0x03;
	p2=0x03;
	p3=0x03;
	for(i=0x00 ; i < outSize ; i++)
	{
		/** Container Length, byte offset 0, 4 bytes **/
		if(i == 0)
			errorCode=FTDI_TX_Byte(outSize,TIMEOUT_DEFAULT);
		else if (i < 4)
			errorCode=FTDI_TX_Byte(0x00,TIMEOUT_DEFAULT);

		/** Container Type, byte offset 4, 2 bytes **/
		else if (i < 5)
			errorCode=FTDI_TX_Byte(PTPcontainerTX.type,TIMEOUT_DEFAULT);
		else if (i < 6)
			errorCode=FTDI_TX_Byte( (PTPcontainerTX.type>>8) ,TIMEOUT_DEFAULT);

		/** Code, byte offset 6, 2 bytes **/
		else if (i < 7)
			errorCode=FTDI_TX_Byte(PTPcontainerTX.code,TIMEOUT_DEFAULT);
		else if (i < 8)
			errorCode=FTDI_TX_Byte( (PTPcontainerTX.code>>8),TIMEOUT_DEFAULT);

		/** Transaction ID, byte offset 8, 4 bytes **/
		else if (i < 9)
			errorCode=FTDI_TX_Byte(PTPcontainerTX.tID,TIMEOUT_DEFAULT);
		else if (i < 12)
			errorCode=FTDI_TX_Byte(0x00,TIMEOUT_DEFAULT);

		/** Parameter 1, byte offset 12, 4 bytes, Send LSB first **/
		else if (i < 16)
		{
			errorCode=FTDI_TX_Byte(PTPcontainerTX.param1[p1],TIMEOUT_DEFAULT);
			p1--;
		}

		/** Parameter 2, byte offset 16, 4 bytes, Send LSB first **/
		else if (i < 20)
		{
			errorCode=FTDI_TX_Byte(PTPcontainerTX.param2[p2],TIMEOUT_DEFAULT);
			p2--;
		}

		/** Parameter 3, byte offset 16, 4 bytes, Send LSB first **/
		else if (i < 24)
		{
			errorCode=FTDI_TX_Byte(PTPcontainerTX.param3[p3],TIMEOUT_DEFAULT);
			p3--;
		}


		/** Check for error in PTP packet TX **/
		if (errorCode != 0x00)
			{
				#ifdef DEBUGoutput
					Debug_Detailed(2,1,'2',errorCode);
				#endif
				return errorCode;
			}
	}
	
	/* Check for FTDI Confirmation Response, read back 0x3E0D from FTDI */
	errorCode=FTDI_3E0D_Check();
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Detailed(2,1,'3',errorCode);
		#endif
		return errorCode;
	}


	return 0x00;
}



/******************************************************************
 * Function: PTP_Container_RX                                     *
 * RX PTP Container from USB device                               *
 ******************************************************************
Error codes:
0x00 = No Error
0x01 = Timeout while reading writing to FTDI, reset recommended
0x02 = Timeout while waiting for FTDI to confirm it received USB data packet
0x03 = USB timeout, times out while waiting for USB response
*/

unsigned char PTP_Container_RX(void)
{
	unsigned char tmp;
	unsigned char tmpInByte;
	unsigned char inSize;
	volatile unsigned char i;

	/* Clear anything in the FTDI output FIFO */
	FTDI_Dummy_RD_Timeout(TIMEOUT_DEFAULT); 

	/* Send device read data to FTDI */
	tmp=FTDI_DRD(&inSize);
	if(tmp != 0x00)
		{
			//Debug
			#ifdef DEBUGoutput
				Debug_Detailed(2,2,'1',tmp);
			#endif
			return tmp; // Return error code
		}


	/*** Load up inPTPcontainer array with PTP packet ***/
	for (i=0x00; i < inSize; i++)
	{

		tmp=FTDI_RX_Byte(&tmpInByte, TIMEOUT_DEFAULT);
		tmpPTPcontainer[i]=tmpInByte;

		if(tmp != 0x00)
		{
			//Debug
			#ifdef DEBUGoutput
				Debug_Detailed(2,2,'2',tmp);
			#endif
			return tmp; // Return error code
		}

	}

	/* Check for FTDI Confirmation Response, read back 0x3E0D from FTDI */
	tmp=FTDI_3E0D_Check();
	if (tmp != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Detailed(2,2,'3',tmp);
		#endif
		return tmp;
	}


	return 0x00;
}

/******************************************************************
 * Function: PTP_OpenSession                                      *
 * Open a PTP USB session                                         *
 ******************************************************************
Error codes:
Passes errors from PTP_Container_TX & PTP_Container_RX 
*/
unsigned char PTP_OpenSession(void)
{
	unsigned char errorCode;

	/*** PTP Open Session ***/
	/* Length              |Type     |Code     |TransID            |Payload            */
	/* 0x10,0x00,0x00,0x00,0x01,0x00,0x02,0x10,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00 */
	PTPcontainerTX.type=PTP_CONTAINER_CMD;
	PTPcontainerTX.code=PTP_OC_OpenSession;
	PTPcontainerTX.numOfParm=0x01;
	PTPcontainerTX.param1[0]=0x00;
	PTPcontainerTX.param1[1]=0x00;
	PTPcontainerTX.param1[2]=0x00;
	PTPcontainerTX.param1[3]=PTPcontainerTX.sessionID;
	errorCode=PTP_Container_TX();
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Detailed(2,3,'O',errorCode);
		#endif
		return errorCode;
	}

	/** Receive data from camera **/
	errorCode=PTP_Container_RX();
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Detailed(2,3,'R',errorCode);
		#endif
		return errorCode;
	}
	
	/* Increment Transaction ID */
	PTPcontainerTX.tID++;

	return 0x00;
}



/******************************************************************
 * Function: PTP_CloseSession                                     *
 * Open a PTP USB session                                         *
 ******************************************************************
Error codes:
Passes errors from PTP_Container_TX & PTP_Container_RX 
*/
unsigned char PTP_CloseSession(void)
{
	unsigned char errorCode;

	/*** PTP Close Session ***/
	/* Length              |Type     |Code     |TransID            |Payload            */
	/* 0x10,0x00,0x00,0x00,0x01,0x00,0x03,0x10,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00 */
	PTPcontainerTX.type=PTP_CONTAINER_CMD;
	PTPcontainerTX.code=PTP_OC_CloseSession;
	PTPcontainerTX.numOfParm=0x01;
	PTPcontainerTX.param1[0]=0x00;
	PTPcontainerTX.param1[1]=0x00;
	PTPcontainerTX.param1[2]=0x00;
	PTPcontainerTX.param1[3]=PTPcontainerTX.sessionID;
	errorCode=PTP_Container_TX();
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Detailed(2,4,'C',errorCode);
		#endif
		return errorCode;
	}

	/** Receive data from camera **/
	errorCode=PTP_Container_RX();
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Detailed(2,4,'R',errorCode);
		#endif
		return errorCode;
	}
	
	/* Increment Transaction ID */
	PTPcontainerTX.tID++;
	PTPcontainerTX.sessionID++;

	return 0x00;
}




/******************************************************************
 * Function: PTP_CANON_SetRemoteMode                              *
 * Set Camera into remote control mode                            *
 ******************************************************************
 Must be used within an open session!!	
*/
unsigned char PTP_CANON_SetRemoteMode(void)
{
	unsigned char errorCode;

	/*** PTP_OC_CANON_EOS_SetRemoteMode, 1=set, 0=clear ***/
	/* Length              |Type     |Code     |TransID            |Payload            */
	/* 0x10,0x00,0x00,0x00,0x01,0x00,0x14,0x91,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00 */
	PTPcontainerTX.type=PTP_CONTAINER_CMD;
	PTPcontainerTX.code=CANON_SetRemoteMode;
	PTPcontainerTX.numOfParm=0x01;
	PTPcontainerTX.param1[0]=0x00;
	PTPcontainerTX.param1[1]=0x00;
	PTPcontainerTX.param1[2]=0x00;
	PTPcontainerTX.param1[3]=0x01;
	errorCode=PTP_Container_TX();
	if (errorCode != 0x00)
		return errorCode;


	/** Receive data from camera **/
	errorCode=PTP_Container_RX();
	if (errorCode != 0x00)
		return errorCode;


	/* Increment PTP Transaction ID */
	PTPcontainerTX.tID++;

	return 0x00;
}


/******************************************************************
 * Function: PTP_CANON_SendOpCode
 * Send operation code to camera
 ******************************************************************
 Must be used within an open session
*/
unsigned char PTP_CANON_SendOpCode(uint16_t OpCode, uint16_t param1)
{
	unsigned char errorCode;

	PTPcontainerTX.type=PTP_CONTAINER_CMD;
	PTPcontainerTX.code=OpCode;
	PTPcontainerTX.numOfParm=0x01;
	PTPcontainerTX.param1[0]=0x00;
	PTPcontainerTX.param1[1]=0x00;
	PTPcontainerTX.param1[2]=param1>>8;
	PTPcontainerTX.param1[3]=param1;
	errorCode=PTP_Container_TX();
	if (errorCode != ECODE_NO_ERROR)
		return errorCode;

	/** Receive data from camera **/
	errorCode=PTP_Container_RX();
	if (errorCode != 0x00)
		return errorCode;

	/* Increment PTP Transaction ID */
	PTPcontainerTX.tID++;

	return ECODE_NO_ERROR;
}



/******************************************************************
 * Function: PTP_CANON_WaitForEvent                               *
 * Wait for specified for camera event                            *
 ******************************************************************
 Must be used within an open session!!	
 Returns if ANY event passed in is found, does not wait for
*/
unsigned char PTP_CANON_WaitForEvent(uint16_t eventCodeToFind)
{
	unsigned char errorCode;
	volatile uint32_t inSize;
	uint16_t eventSize;
	unsigned char eventFound;
	volatile uint16_t eventCounter;
	volatile uint32_t i;
	unsigned char tmpInByte1;
	uint16_t eventCode;

	eventCode=0x0000;
	eventFound=FALSE_B;

	/* keep looking for event code until eventFound is set */
	while (eventFound == FALSE_B)
	{

		#ifdef DEBUG_WaitForEvent_ALL_TRAFFIC_OUT_SELECTIVE
			if(debugWaitForEventAllIO == TRUE_B)
				debugIOenableASCII=TRUE_B;
		#endif

		/*** PTP_OC_CANON_EOS_GetEvent ***/
		PTPcontainerTX.type=PTP_CONTAINER_CMD;
		PTPcontainerTX.code=CANON_GetEvent;
		PTPcontainerTX.numOfParm=0x00;
		errorCode=PTP_Container_TX();
		if (errorCode != ECODE_NO_ERROR)
		{
			#ifdef DEBUGoutput
				Debug_Detailed(2,6,'1',errorCode);
			#endif
			return errorCode;
		}

		/* Increment PTP Transaction ID */
		PTPcontainerTX.tID++;

		delay_ms(USB_WRITE_TO_READ_DELAY); /* wait before reading USB device \ camera */


		#ifdef DEBUG_WaitForEvent_ALL_TRAFFIC_OUT_SELECTIVE
			if(debugWaitForEventAllIO == TRUE_B)
			{
				Debug_Out(0x0D);
			}
		#endif
		
		/* Open USB Read Session & get total size of event packet */
		errorCode=USB_RX_Byte_StartSession(&inSize);
		if(errorCode != ECODE_NO_ERROR)
		{
			/* Used for Debug Only */
			#ifdef DEBUGoutput
				Debug_Detailed(2,6,'2',errorCode);
			#endif
			return errorCode; /* Return error code */
		}


		/**** Strip PTP packet off input data, first 8 (12-4 bytes of length already read) bytes  ****/
		for (i=0x00000000 ; ((i < 0x00000008) && (i < inSize)) ; i++)	
		{
			errorCode=USB_RX_Byte(&tmpInByte1);

			if(errorCode != ECODE_NO_ERROR)
			{
				/* Debug */
				#ifdef DEBUGoutput
					Debug_Detailed(2,6,'3',errorCode);
				#endif
				return errorCode;
			}
		}

		#ifdef DEBUG_WaitForEvent_ALL_TRAFFIC_OUT_SELECTIVE
			if(debugWaitForEventAllIO == TRUE_B)
			{
				Debug_Out(0x0D);
			}
		#endif
		#ifdef DEBUG_TX_EVENT_CODES
			if(i == inSize)
				Debug_Print_ASCII_String("No Event Data");
		#endif

		eventSize=0x0000;			/* reset event size */
		eventCounter=0x0000; 	/* reset counter */

		/**** Process event data from camera ****/
		for ( ; (i < inSize) ; i++)	
		{
			/** If reading first byte of event data chunk load eventSize var.  
			    Or if current event chunk has been fully read move on to next event **/
			if(eventCounter == eventSize)
			{

			#ifdef DEBUG_WaitForEvent_ALL_TRAFFIC_OUT_SELECTIVE
				if(debugWaitForEventAllIO == TRUE_B)
				{
					Debug_Out(0x0D);
					Debug_Out(0x0D);
				}
			#endif

				/* Load first byte of event size and increment counter */
				errorCode=USB_RX_Byte(&tmpInByte1);
				DEBUG_DETAILED_OUTPUT_W_IF(2,6,'4',errorcode);
				
				eventSize = (uint16_t)tmpInByte1;
				i++;

				/* if no error and not at the end of the dataset, load the next byte */
				if ((errorCode == ECODE_NO_ERROR) && (i < inSize))
				{
					errorCode=USB_RX_Byte(&tmpInByte1);
					DEBUG_DETAILED_OUTPUT_W_IF(2,6,'5',errorcode);

					eventSize = eventSize | ( ((uint16_t)tmpInByte1)<<0x08 );
				}

				eventCounter=0x0001; /* reset counter to 1 because two bytes have been read */
			}
			/** dump the rest of the event size 16-bits of the 32-bit event size block **/
			else if (eventCounter < 0x0004)
			{
				errorCode=USB_RX_Byte(&tmpInByte1);
				DEBUG_DETAILED_OUTPUT_W_IF(2,6,'6',errorcode);
			}
			/** Load first byte of event code **/
			else if (eventCounter < 0x0005)
			{
				errorCode=USB_RX_Byte(&tmpInByte1);
				DEBUG_DETAILED_OUTPUT_W_IF(2,6,'7',errorcode);
				
				eventCode = (uint16_t)tmpInByte1;
			}
			/** Load second byte of event code **/
			else if (eventCounter < 0x0006)
			{
				errorCode=USB_RX_Byte(&tmpInByte1);
				DEBUG_DETAILED_OUTPUT_W_IF(2,6,'8',errorcode);
				
				eventCode |= ( ((uint16_t)tmpInByte1)<<0x08 );

				/* set eventFound if event code matches eventCodeToFind */
				/* while loop will not exit until all data is dumped from get event CMD*/
				if(eventCode == eventCodeToFind)
					eventFound=TRUE_B;

				/** Debug Output **/
				#ifdef DEBUG_TX_EVENT_CODES
					//Debug_ASCII('Event: ');
					Debug_Out_HEX_to_ASCII(eventCode>>8);
					Debug_Out_HEX_to_ASCII_W_CR(eventCode);
				#endif

			}
			/** Dump last two byes of event code **/
			else if (eventCounter < 0x0008)
			{
				errorCode=USB_RX_Byte(&tmpInByte1);
				DEBUG_DETAILED_OUTPUT_W_IF(2,6,'9',errorcode);
			}
			else if (eventCounter >= 0x0008)
			{
				/* dump next byte */
				errorCode=USB_RX_Byte(&tmpInByte1);
				DEBUG_DETAILED_OUTPUT_W_IF(2,6,'A',errorcode);		
			}

			if(errorCode != ECODE_NO_ERROR)
				return errorCode;

			eventCounter++; /* increment event code counter */

		} /*** End of for loop ***/

		/** Debug Output **/
		#ifdef DEBUG_TX_EVENT_CODES
			//Debug_ASCII('Event not found, checking again ');
			Debug_Out(0x0D); /* Output another end of line to seperate each check*/
		#endif

		#ifdef DEBUG_WaitForEvent_ALL_TRAFFIC_OUT_SELECTIVE
			if(debugWaitForEventAllIO == TRUE_B)
			{
				Debug_Out(0x0D);
			}
		#endif

		/** Receive confirmation PTP packet data from camera **/
		/** This is a PTP packet with a response code **/
		errorCode=PTP_Container_RX();
		if (errorCode != ECODE_NO_ERROR)
		{
			#ifdef DEBUGoutput
				Debug_Detailed(2,6,'B',errorCode);
			#endif
			return errorCode;
		}

		#ifdef DEBUG_WaitForEvent_ALL_TRAFFIC_OUT_SELECTIVE
			if(debugWaitForEventAllIO == TRUE_B)
			{
				debugIOenableASCII=FALSE_B;
				Debug_Out(0x0D);
				Debug_Out(0x0D);
			}
		#endif

/********************* ADD PTP ERROR CHECK ******************************/

		/* Delay before checking camera again */
		delay_ms(PTP_CANON_WaitForEvent_DELAY);


	}/*** End of eventFound while loop ***/

	return ECODE_NO_ERROR;
}


/******************************************************************
 * Function: PTP_CANON_SetExposure                                *
 * Set Canon exposure value                                       *
 ******************************************************************
 Must be used within an open session & remote mode
*/
unsigned char PTP_CANON_SetExposure(unsigned char exposureValue)
{
	unsigned char errorCode;

	/*** PTP_OC_CANON_EOS_SetDevicePropValueEx ***/
	PTPcontainerTX.type=PTP_CONTAINER_CMD;
	PTPcontainerTX.code=CANON_SetDevicePropValueEx;
	PTPcontainerTX.numOfParm=0x00;
	errorCode=PTP_Container_TX();
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Print('a', errorCode);
		#endif
		return errorCode;
	}

	/*** PTP_OC_CANON_EOS_SetDevicePropValueEx, Data ***/
	/* Length      |Type |Code |TransID    |Parm1      |Param2     |Param3       */
	/* 18 00 00 00 02 00 10 91 51 00 00 00 0C 00 00 00 02 D1 00 00 68 00 00 00   */
	PTPcontainerTX.type=PTP_CONTAINER_DATA;
	PTPcontainerTX.code=CANON_SetDevicePropValueEx;
	PTPcontainerTX.numOfParm=0x03;
	PTPcontainerTX.param1[0]=0x00;
	PTPcontainerTX.param1[1]=0x00;
	PTPcontainerTX.param1[2]=0x00;
	PTPcontainerTX.param1[3]=0x0C;
	PTPcontainerTX.param2[0]=0x00;
	PTPcontainerTX.param2[1]=0x00;
	PTPcontainerTX.param2[2]=0xD1;
	PTPcontainerTX.param2[3]=0x02;
	PTPcontainerTX.param3[0]=0x00;
	PTPcontainerTX.param3[1]=0x00;
	PTPcontainerTX.param3[2]=0x00;
	PTPcontainerTX.param3[3]=exposureValue;
	errorCode=PTP_Container_TX();
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Print('b', errorCode);
		#endif
		return errorCode;
	}

	/** Receive data from camera **/
	errorCode=PTP_Container_RX();
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Print('c', errorCode);
		#endif
		return errorCode;
	}

	/* Increment Transaction ID */
	PTPcontainerTX.tID++;

return 0x00;
}


/******************************************************************
 * Function: PTP_CANON_TakePicture                                *
 * Use MTP commands to take a picture on canon brand DSLRs        *
 ******************************************************************
 Input Arguments
 - unsigned char singleOpp: used to tell this function whether to open and close
    a PTP/MTP session before and after taking a picture.
	 Set to 0xFF for true  (open and close a session)
	 Set to 0x00 for false, used when taking a picture is part of another sequence of cmds

	
 Error codes:
 0x00 = No Error

*/
unsigned char PTP_CANON_TakePicture()
{
	unsigned char errorCode;


	/*** PTP_OC_CANON_EOS_RemoteRelease ***/  
	/* Length              |Type     |Code     |TransID            |Payload */
	/* 0x0C,0x00,0x00,0x00,0x01,0x00,0x0F,0x91,0x02,0x00,0x00,0x00          */
	PTPcontainerTX.type=PTP_CONTAINER_CMD;
	PTPcontainerTX.code=CANON_RemoteRelease;
	PTPcontainerTX.numOfParm=0x00;
	errorCode=PTP_Container_TX();
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Print('d', errorCode);
		#endif
		return errorCode;
	}

	/** Receive data from camera **/
	errorCode=PTP_Container_RX();
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Print('e', errorCode);
		#endif
		return errorCode;
	}

	/* Increment Transaction ID */
	PTPcontainerTX.tID++;


	/* Wait for shoot to finish */
	errorCode=PTP_CANON_WaitForEvent(PTP_EC_CANON_EOS_ObjectAddedEx);
	if (errorCode != 0x00)
	{
		#ifdef DEBUGoutput
			Debug_Print('f', errorCode);
		#endif
		return errorCode;
	}
	

	return 0x00;

}



/******************************************************************
 * Function: PTP_CANON_BraketSeq_Man                              *
 * Use MTP commands to take a bracketed set of picture on a cannon  *
 * brand DSLRs.  Manual setting the exposure time on each shot    *
 ******************************************************************
 Input Arguments
 - unsigned char numOfShots: Number of shots, must be odd
 - unsigned char zeroEV: zero exposure value, bracket center point

 Error codes:
 0x00 = No Error
*/
unsigned char PTP_CANON_BraketSeq_Man(unsigned char numOfShots,unsigned char zeroEV)
{

	unsigned char errorCode;
	unsigned char i;



	// zeroEV=0x6D; /* For Testing */
	for(i=0 ; i < numOfShots ; i++)
	{
		/*** Set Exposure ***/
		errorCode=PTP_CANON_SetExposure(zeroEV);
		if (errorCode != 0x00)
			{
				#ifdef DEBUGoutput
					Debug_Print('c', errorCode);
				#endif
				return errorCode;
			}


		/*** Take Picture ***/
		errorCode=PTP_CANON_TakePicture();
		if (errorCode != 0x00)
		{
			#ifdef DEBUGoutput
				Debug_Print('f', errorCode);
			#endif
			return errorCode;
		}

		zeroEV=zeroEV-0x08;
	}


	

	return 0x00;
}




