/*************************************************************
	File:		mainAVR.c
	Project:	USB PTP\MTP Camera Controller

	Target:	ATmega644 on STK600
	IDE:		AVR studio 4.18 SP2

	Program Version: 0.01

	Author:				Luke Skaff
	Author's Email:	Luke.Skaff@gmail.com
	Author's Website:	http://blog.LukeSkaff.com/
							http://www.LukeSkaff.com/
 ************************************************************/


/******************************************************************************
	Program description
 This program controls a DLSR camera using a AVR or Nintendo DS connected to a FTDI Vinculum-I
 USB Host controller.
 
 Important code viewing note:
 The tab spacing (tab width) in my code editor (AVR studio) is set to 3 space per tab, if your editor is not setup this way
 the code will be much harder to read with incorrect indents everywhere.

 Current limitations with code:
 - It has been some time since I have compiled this code on the Nintendo DS
   platform so it will probably need to be reworked slightly to work on that. Mainly the HDR bracket
   function which uses a AVR specif lookup table.
 - The HDR bracket function needs some work to implement the shutter speed lookup table at the top of PTP_basic.c   
 - Only works with Canon DSLRs, I have not implemented the standard PTP commands needed for Nikon's.
   This should not be to hard to implement since all the basic PTP/MTP framework has be taken care of.
 - I started to implement the needed to code to run on a Xmega processor, not finished.


 Current Issues with code:
 - My Canon 1000D (Rebel XS) will not accept any commands after three open and closings of a PTP/MTP
   session.  The simple fix for this is just open one PTP/MTP session and do not close it unless you are going
   disconnect the camera.
 ******************************************************************************/


#include "mainAVR.h"


/******************************************************************
  Function: main
  Main control function
 ******************************************************************/
int main (void)
{

	/*** Setup pins as inputs\outputs ***/
#ifdef USE_AVR_MEGA
	DDRA = 0x00;	// Configure PORTA as an input
	DDRB = 0x00;	// Configure PORTB as an input
	DDRC = 0xFF;	// Configure PORTC as an output
	DDRD = (0<<PD0)|(1<<PD1)|(0<<PD2)|(0<<PD3)|(1<<PD4)|(1<<PD5)|(1<<PD6)|(1<<PD7);  // Configure PORTD
	PORTD = 0x00; 	//Disable pullups 
#endif
#ifdef USE_AVR_XMEGA
	PORTA.DIR = 0x00; /* Configure PORTA as an input */
	PORTB.DIR = 0x00; /* Configure PORTA as an input */
	PORTC.DIR = 0xFF; /* Configure PORTA as an output */
	PORTD.DIR = (0<<PIN0)|(1<<PIN1)|(0<<PIN2)|(0<<PIN3)|(1<<PIN4)|(1<<PIN5)|(1<<PIN6)|(1<<PIN7);
#endif

	/* Initialize FTDI port pins */
	bit_Set(FTDI_CTRL_PORT,FTDI_RD_);
	bit_Clear(FTDI_CTRL_PORT,FTDI_WR);

	USART_init(); /* Initialize USART */

	/* Print Start to UART (terminal) */
	USART_Transmit('S');
	USART_Transmit('t');
	USART_Transmit('a');
	USART_Transmit('r');
	USART_Transmit('t');
	USART_Transmit(0x0D); /* carriage return */
	USART_Transmit(0x0D); /* carriage return */
	

	unsigned char FirstPress=TRUE_B;
	unsigned char errorCode;
	unsigned int count= 0;

	
	while (1)
	{

		/*** Flash all LEDs ****/
		if(count < 0x7FFF)
			LEDPORT=0x00; /* Turn on all LEDS */
		else
			LEDPORT=0xFF; /* Turn off all LEDS */

	
		if( (PINB != 0xFF) && (FirstPress == TRUE_B))
		{
			LEDPORT=0xFF; /* Turn off all LEDS */

			FirstPress=FALSE_B;
			delay_ms(10U); /* Allow switches to debounce and stabilize */

			switch(PINB)
			{

				/**** Switch 0 ****/
				case 0b11111110:
				{
					bit_Clear(PORTC,0); //Turn on LED 0

					/**** Must be pressed first, initializes the FTDI Vinculum ****/

					errorCode=FTDI_init();
					#ifdef DEBUGoutput
						if (errorCode != 0x00)
							Debug_Detailed(9,0,'1',errorCode);
					#endif


					bit_Set(PORTC,0); //Turn off LED 0
				}
					break;



				/**** Switch 1 ****/
				case 0b11111101:
				{
					bit_Clear(PORTC,1); /* Turn on LED 1 */

					/**** Opens PTP/MTP session with camera ****/

					/*** PTP Open Session ***/
					errorCode=PTP_OpenSession();
					#ifdef DEBUGoutput
						if (errorCode != ECODE_NO_ERROR)
							Debug_Detailed(0,0,'1',errorCode);
					#endif

					/*** PTP Set Canon Remote Mode ***/
					errorCode=PTP_CANON_SendOpCode(PTP_OC_CANON_SetRemoteMode,PTP_OC_CANON_SetRemoteMode_SET);
					#ifdef DEBUGoutput
						if (errorCode != ECODE_NO_ERROR)
							Debug_Detailed(0,0,'2',errorCode);
					#endif

					/*** PTP Set Canon Event Mode ***/
					errorCode=PTP_CANON_SendOpCode(PTP_OC_CANON_EOS_SetEventMode,PTP_OC_CANON_EOS_SetEventMode_SET);
					#ifdef DEBUGoutput
						if (errorCode != ECODE_NO_ERROR)
							Debug_Detailed(0,0,'3',errorCode);
					#endif

					/*** Dump Events ***/
					errorCode=PTP_CANON_WaitForEvent(PTP_EC_CANON_EOS_PropValueChanged);
					#ifdef DEBUGoutput
						if (errorCode != ECODE_NO_ERROR)
							Debug_Detailed(0,0,'4',errorCode);
					#endif


					bit_Set(PORTC,1); /* Turn off LED 1 */
				}
					break;
			

				/**** Switch 2 ****/
				case 0b11111011:
				{
					bit_Clear(PORTC,2); /* Turn on LED 2 */

					/**** Closes PTP/MTP session ****/

					/*** PTP Clear Canon Event Mode ***/
					errorCode=PTP_CANON_SendOpCode(PTP_OC_CANON_EOS_SetEventMode,PTP_OC_CANON_EOS_SetEventMode_CLEAR);
					#ifdef DEBUGoutput
						if (errorCode != ECODE_NO_ERROR)
							Debug_Detailed(9,0,'3',errorCode);
					#endif

					/*** PTP Clear Canon Remote Mode ***/
					errorCode=PTP_CANON_SendOpCode(PTP_OC_CANON_SetRemoteMode,PTP_OC_CANON_SetRemoteMode_CLEAR);
					#ifdef DEBUGoutput
						if (errorCode != ECODE_NO_ERROR)
							Debug_Detailed(9,0,'2',errorCode);
					#endif

					/*** PTP Close Session ***/
					errorCode=PTP_CloseSession();
					#ifdef DEBUGoutput
						if (errorCode != ECODE_NO_ERROR)
							Debug_Detailed(9,0,'1',errorCode);
					#endif


					bit_Set(PORTC,2); /* Turn off LED 2 */

				}
				break;


				/**** Switch 3 ****/
				case 0b11110111:
				{
					bit_Clear(PORTC,3); /* Turn on LED 3 */

						debugWaitForEventAllIO = TRUE_B;

					bit_Set(PORTC,3); /* Turn off LED 3 */
				}
					break;



				/**** Switch 4 ****/
				case 0b11101111:
				{
					bit_Clear(PORTC,4); /* Turn on LED 4 */

					Debug_Print_HEX_to_ASCII(0x00);
					USART_Transmit(0x0D);
					Debug_Print_HEX_to_ASCII(0x09);
					USART_Transmit(0x0D);
					Debug_Print_HEX_to_ASCII(0x0A);
					USART_Transmit(0x0D);
					Debug_Print_HEX_to_ASCII(0x0F);
					USART_Transmit(0x0D);

					Debug_Print_HEX_to_ASCII(0x10);
					USART_Transmit(0x0D);
					Debug_Print_HEX_to_ASCII(0x90);
					USART_Transmit(0x0D);
					Debug_Print_HEX_to_ASCII(0xA0);
					USART_Transmit(0x0D);
					Debug_Print_HEX_to_ASCII(0xF0);
					USART_Transmit(0x0D);

					Debug_Print_HEX_to_ASCII(0x11);
					USART_Transmit(0x0D);
					Debug_Print_HEX_to_ASCII(0x99);
					USART_Transmit(0x0D);
					Debug_Print_HEX_to_ASCII(0xAA);
					USART_Transmit(0x0D);
					Debug_Print_HEX_to_ASCII(0xFF);
					USART_Transmit(0x0D);

					bit_Set(PORTC,4); /* Turn off LED 4 */

				}
				break;


				/**** Switch 5 ****/
				case 0b11011111:
				{
					bit_Clear(PORTC,5); /* Turn on LED5 */


					errorCode=PTP_CANON_WaitForEvent(PTP_EC_CANON_EOS_PropValueChanged);
					#ifdef DEBUGoutput
						if (errorCode != 0x00)
							Debug_Detailed(9,0,'5',errorCode);
					#endif
					

					bit_Set(PORTC,5); /* Turn off LED 5 */
				}
				break;

				
				/**** Switch 6 ****/
				case 0b10111111:
				{
					bit_Clear(PORTC,6); /* Turn on LED 6 */

					/**** Take HDR bracket set of pictures ****/

					errorCode=PTP_CANON_BraketSeq_Man(0x07,0x6D);
					#ifdef DEBUGoutput
						if (errorCode != 0x00)
							Debug_Detailed(9,0,'6',errorCode);
					#endif

					bit_Set(PORTC,6); /* Turn off LED 6 */
				}
				break;


				case 0b01111111: //switch 7
				{
					bit_Clear(PORTC,7); //Turn on LED 7

					/**** Take a Picture ****/

					errorCode=PTP_CANON_TakePicture();
					#ifdef DEBUGoutput
						if (errorCode != 0x00)
							Debug_Detailed(9,0,'7',errorCode);
					#endif

					bit_Set(PORTC,7); //Turn off LED 7
				}
				break;





				default:
					break;
			}
		}
		else if(PINB == 0xFF) 
			FirstPress=TRUE_B;



		count++;
		_delay_us(10);
   }

} /* End of main function */
