/*************************************************************
	File:		main.c
	Project:	USB PTP\MTP Camera Controller

	Target:	Nintendo DS
	IDE:		devkitPro - PAlib

	Author:				Luke Skaff
	Author's Email:	Luke.Skaff@gmail.com
	Author's Website:	http://blog.LukeSkaff.com/
							http://www.LukeSkaff.com/
 ************************************************************/
#include "main.h"


#define WAIT_CR_REG (*(vu16*)0x04000204)
/* WAITCNT \ WAIT_CR_REG 0x04000204
 http://nocash.emubase.de/gbatek.htm#dsmemorycontrolcartridgesandmainram
 4000204h - NDS9 - EXMEMCNT - 16bit - External Memory Control (R/W)
 4000204h - NDS7 - EXMEMSTAT - 16bit - External Memory Status (R/W..R)

  0-1   32-pin GBA Slot SRAM Access Time    (0-3 = 10, 8, 6, 18 cycles)
  2-3   32-pin GBA Slot ROM 1st Access Time (0-3 = 10, 8, 6, 18 cycles)
  4     32-pin GBA Slot ROM 2nd Access Time (0-1 = 6, 4 cycles)
  5-6   32-pin GBA Slot PHI-pin out   (0-3 = Low, 4.19MHz, 8.38MHz, 16.76MHz)
  7     32-pin GBA Slot Access Rights     (0=ARM9, 1=ARM7)
  8-10  Not used (always zero)
  11    17-pin NDS Slot Access Rights     (0=ARM9, 1=ARM7)
  12    Not used (always zero)
  13    Not used (always set ?)
  14    Main Memory Interface Mode Switch (0=Async/GBA/Reserved, 1=Synchronous)
  15    Main Memory Access Priority       (0=ARM9 Priority, 1=ARM7 Priority)

  Default WAIT_CR on load of custom .nds off R4 cartridge
  11101 000 1 00 0 00 00


  WAIT_CR_REG |= (1<<14)|(0<<6)|(1<<5)|(0<<4)|(1<<3)|(1<<2)|(1<<1)|(1<<0); //Set slowist write possible with clock, sync
  WAIT_CR_REG |= (1<<14)|(0<<6)|(0<<5)|(0<<4)|(1<<3)|(1<<2)|(1<<1)|(1<<0); //Set slowist write possible

Bit 0-6 can be changed by both NDS9 and NDS7, changing these bits affects the local EXMEM register only, not that of the other CPU.
Bit 7-15 can be changed by NDS9 only, changing these bits affects both EXMEM registers, ie. both NDS9 and NDS7 can read the current NDS9 setting.
Bit 14=0 is intended for GBA mode, however, writes to this bit appear to be ignored?
*/

/* External Memory (Game Pak)
  08000000-09FFFFFF   Game Pak ROM/FlashROM (max 32MB) - Wait State 0
  0A000000-0BFFFFFF   Game Pak ROM/FlashROM (max 32MB) - Wait State 1
  0C000000-0DFFFFFF   Game Pak ROM/FlashROM (max 32MB) - Wait State 2
  0E000000-0E00FFFF   Game Pak SRAM    (max 64 KBytes) - 8bit Bus width
*/

/* Writing to eeprom, ONLY works in GBA mode on DS
http://nocash.emubase.de/gbatek.htm#gbacartbackupeeprom
*/


void Debug_Pause()
{
	PA_WaitForVBL();
	PA_OutputSimpleText(0, 0, 20, "Press Any Key... ");
	PA_WaitForVBL();

	while(1)
	{
		PA_WaitForVBL();

		if(Pad.Newpress.Anykey)
		{
			PA_OutputSimpleText(0, 0, 20, "                 ");
			PA_WaitForVBL();
			return;
		}
	}
}

void Debug_Print(char error, int errorCode)
{
	PA_WaitForVBL();

	char errorArray[2];
	errorArray[0]=error;
	errorArray[1]=':';
	PA_OutputText(0, 0, 2, "ERROR %s %d     ",errorArray, errorCode);
	Debug_Pause();
}






int main(){
	// Initialize PAlib
	PA_Init();

	// Enable the ARM9 to access the GBA cartridge memory area
    WAIT_CR_REG &= ~0x0080;

	bool RUN = false;

	PA_InitVBL(); // Initializes a standard VBL

	PA_InitText(0, 2); //screen (0 or 1), background number (0-3)
	PA_InitText(1, 2); //screen (0 or 1), background number (0-3)
	PA_OutputSimpleText(1, 1, 2, "Memtest, hit a Key");  //screen, x coordinate(0-31), y coordinate (0-19), text string

	int size=0;
	
	vu16 DummyVar16b = 0xF0F0;
	vu8 DummyVar8b = 0x00;
	vu8 DummyVar8b2 = 0x00;
	u8 tempVar8b=0x00;
	char tempCharArray[4]= "    ";

	// Set addresses
	vu16 *addrPtr;  //define's a pointer variable
	addrPtr = (vu16 *) 0xA000000;  //set the pointer's address

	vu16 *addrPtr2;  //define's a pointer variable
	addrPtr2 = (vu16 *) 0xA000016;  //set the pointer's address

	vu16 *addrPtr3;  //define's a pointer variable
	addrPtr3 = (vu16 *) 0x8020000;  //set the pointer's address

	vu16 *addrPtr4;  //define's a pointer variable
	addrPtr4 = (vu16 *) 0x8000000;  //set the pointer's address

	unsigned char tmp;
	unsigned char errorCode;
	bool writeSet = true;

	/* Infinite main program loop */
	while(true){

		/* Setup GBA bus speed, press A first and press X*/
		if(Pad.Newpress.A || RUN)
		{
			writeSet = true;
			PA_OutputSimpleText(1, 1, 2, "Press X or Y           ");
			while(writeSet)
			{
				if(Pad.Newpress.X)
				{
					PA_OutputSimpleText(1, 1, 2, "sync default      ");
					WAIT_CR_REG |= (1<<14)|(0<<6)|(1<<5)|(0<<4)|(1<<3)|(1<<2)|(1<<1)|(1<<0); //Set slowist write possible
					writeSet=false;
				}
				if(Pad.Newpress.Y)
				{
					PA_OutputSimpleText(1, 1, 2, "async            ");
					WAIT_CR_REG |= (0<<14)|(0<<6)|(1<<5)|(0<<4)|(1<<3)|(1<<2)|(1<<1)|(1<<0); //Set slowist write possible
					writeSet=false;
				}
			PA_WaitForVBL();
			}

		}

		if(Pad.Newpress.B || RUN)
		{
			PA_OutputSimpleText(1, 1, 2, "Take Picture             ");
			PA_WaitForVBL();

			/*** PTP Open Session ***/
			errorCode=PTP_OpenSession();
			if(errorCode != 0x00)
			{
				Debug_Print('O', errorCode);
			}

			/*** PTP Set Canon Remote Mode ***/
			errorCode=PTP_CANON_SendOpCode(PTP_OC_CANON_SetRemoteMode,PTP_OC_CANON_SetRemoteMode_SET);
			if(errorCode != 0x00)
			{
				Debug_Print('R', errorCode);
			}

			/*** PTP Set Canon Event Mode ***/
			errorCode=PTP_CANON_SendOpCode(PTP_OC_CANON_EOS_SetEventMode,PTP_OC_CANON_EOS_SetEventMode_SET);
			if(errorCode != 0x00)
			{
				Debug_Print('E', errorCode);
			}

			/*** Dump Events ***/
			errorCode=PTP_CANON_WaitForEvent(PTP_EC_CANON_EOS_PropValueChanged);
			if(errorCode != 0x00)
			{
				Debug_Print('D', errorCode);
			}

	
		}

		if(Pad.Newpress.Y || RUN)
		{
			PA_OutputSimpleText(1, 1, 2, "Y read from 0x8000000 ");
			DummyVar16b = *(vu16 *) 0x8000000;
		}

		if(Pad.Newpress.X || RUN)
		{
			PA_OutputSimpleText(1, 1, 2, "X read 16b 0x8000002 ");
			DummyVar16b = *(vu16 *)0x8000002;
		}

		if(Pad.Newpress.Up || RUN)
		{
			addrPtr4 = (vu16 *) 0x8000000;  //set the pointer's address
			size=5;
			PA_OutputSimpleText(1, 1, 2, "Up write 0xFFFF to 0x8000000 ");
			while(size--)
				*addrPtr4++ = 0xEAAB;
		}

		if(Pad.Newpress.Down || RUN)
		{
			addrPtr2 = (vu16 *) 0x8000000;  //set the pointer's address
			size=5;
			PA_OutputSimpleText(1, 1, 2, "Down Read five from 0x8000000 ");
			while(size--)
			{
				DummyVar16b = *addrPtr2++;
			}
			*(vu16 *)0x8000000 = addrPtr2;
			//PA_OutputText(1, 0, 0,"test %d", addrPtr2);

		}

		if(Pad.Newpress.Left || RUN)
		{
			PA_OutputSimpleText(1, 1, 2, "Left read 16 0x8000006 ");
			DummyVar8b = *(vu16 *)0x8000006;
		}

		if(Pad.Newpress.Right || RUN)
		{
			PA_OutputSimpleText(1, 1, 2, "clear reg, output  ");
			FTDI_CTRL_REG=0b0000000000000000;
		}


		if(Pad.Newpress.Start)
		{
			PA_OutputSimpleText(1, 1, 2, "Init FTDI   ");
			PA_WaitForVBL();

			tempVar8b=FTDI_init();
			if(errorCode != 0x00)
			{
				Debug_Print('I', errorCode);
			}
		}
		
		if(Pad.Newpress.Select)
		{
			PA_OutputSimpleText(1, 1, 2, "Do Nothing ");
			PA_WaitForVBL();

			DummyVar16b = (*(vu16*)0x8000006);
			DummyVar16b = DummyVar16b | 0b0000000000001000;
			(*(vu16*)0x8000006)=DummyVar16b;

			//DummyVar16b = (*(vu16*)0x8000006);
			DummyVar16b = DummyVar16b & ~0b0000000000001000;
			(*(vu16*)0x8000006)=DummyVar16b;

			//bit_Set(FTDI_CTRL_REG,FTDI_RD); 
			//bit_Clear(FTDI_CTRL_REG,FTDI_RD); 
		}
		if(Pad.Newpress.L)
		{
			PA_OutputSimpleText(1, 1, 2, "TX         ");
			PA_WaitForVBL();
/*
			tempVar8b=Debug_Out(0xDE);
			if(tempVar8b != 0x00)
			{
				Debug_Print('D', tempVar8b);
			}
*/
			tempVar8b=FTDI_TX_Byte_NoWait(0xDE);
			if(errorCode != 0x00)
			{
				Debug_Print('T', errorCode);
			}

		}
		if(Pad.Newpress.R)
		{
			PA_OutputSimpleText(1, 1, 2, "RX        ");
			PA_WaitForVBL();

			tempVar8b=FTDI_RX_Byte_NoWait(&DummyVar8b);
			tempCharArray[0] = (char)DummyVar8b;

			if (tempVar8b == 0x00)
			{
				PA_OutputText(1, 1, 2, "text: %s", tempCharArray);
				PA_OutputText(1, 1, 5, "text: %d", (int)DummyVar8b);
			}
			else
				PA_OutputText(1, 1, 2, "ERROR: %d", (int)tempVar8b);
		}

		// Wait until the next frame.
		// The DS runs at 60 frames per second.
		PA_WaitForVBL();
	}
}


