/*************************************************************
	File:		terminal.h
	Project:	USB PTP\MTP Camera Controller

	Author:				Luke Skaff
	Author's Email:	Luke.Skaff@gmail.com
	Author's Website:	http://blog.LukeSkaff.com/
							http://www.LukeSkaff.com/
 ************************************************************/

#include "terminal.h"



/******************************************************************
*******************************************************************
*******************************************************************
Start of platform specific functions
*******************************************************************
*******************************************************************
*******************************************************************/


/**********************************************************************
  Function: USART_init
  Enable and Initialize UART
***********************************************************************/
void USART_init(void)
{
   // Set the USART baudrate registers
	UBRR0L = BAUD_PRESCALE; // Load lower 8-bits of the baud rate value into the low byte of the UBRR register
	UBRR0H = (BAUD_PRESCALE >> 8); // Load upper 8-bits of the baud rate value into the high byte of the UBRR register 

	/*
	UCSRnA - USART Control and Status Register n A
   Enable 2x speed change, 0=16 divider 1=8 divider
   
	  Bit 7 MSB
	R RXCn  -> USART Receive Complete
	0 TXCn  -> TXCn: USART Transmit Complete
	R UDREn -> USART Data Register Empty
	R FEn   -> Frame Error
	R DORn  -> Data OverRun
	R UPEn  -> USART Parity Error
	0 U2Xn  -> Double the USART Transmission Speed, 0=16 divider 1=8 divider
	0 MPCMn -> Multi-processor Communication Mode
	  Bit 0 LSB
	*/
	UCSR0A = (1<<U2X0);


	/*
   UCSRnB - USART Control and Status Register n B

	Bit 7 MSB
	RXCIE0 -> Receive Interrupt 0=disabled
	TXCIE0 -> Transmit Interrupt 0=disabled
	UDRIE0 -> Data register empty interrupt 0=disabled
	RXEN0  = 1 Receive enabled
	TXEN0  = 1 Transmit enabled
	UCSZ02 -> character size
	RXB80  = 0 9'th RX data bit when using 9-bit UART
	TXB80  = 0 9'th TX data bit when using 9-bit UART
	Bit 0 LSB
	*/
	UCSR0B =(0<<RXEN0)|(1<<TXEN0); /* Disable receiver, Enable Transmit */


	/*
	UCSRnC - USART Control and Status Register n C
   
	Bit 7 MSB
	UMSELn1 | 0           | 0          | 1             | 1
	UMSELn0 | 0 Async     | 1 Sync     | 0 Reserved    | 1 Master SPI
	UPMn1   | 0           | 0          | 1             | 1
	UPMn0   | 0 No parity | 1 Reserved | 0 Even Parity | 1 Odd Parity
	USBSn   | Stop Bits, 0=1 stop bits, 1=2 stop bits
	UCSZn1  |
	UCSZn0  | Character size UCSZn0=0,UCSZn1=1,UCSZn0=1 --> 8-bit
	UCPOLn  | 0 = RX'ed data sampled on falling edge, TX data changed on Rising
	Bit 0 LSB
	*/

	/* Set the USART to asynchronous at 8 bits no parity and 1 stop bits */
	UCSR0C =(0<<UMSEL01)|(0<<UMSEL00)|(0<<UPM01)|(0<<UPM00)|(0<<USBS0)|(1<<UCSZ01)|(1<<UCSZ00)|(0<<UCPOL0);
         
}

/******************************************************************
  Function: USART_Transmit
  Transmit byte out UART
 ******************************************************************/
void USART_Transmit(unsigned char data)
{
	// Wait for empty transmit buffer
	while ( !( UCSR0A & (1<<UDRE0)) );

	// Put data into buffer, sends the data
	UDR0 = data;
}


/******************************************************************
*******************************************************************
*******************************************************************
 End of platform specific functions
*******************************************************************
*******************************************************************
*******************************************************************/



void USART_Transmit16b(uint16_t data)
{
	/* Transmits MSB first */
	USART_Transmit(data>>8);
	USART_Transmit(data);
}



void USART_Transmit32b(uint32_t data)
{
	/* Transmits MSB first */
	USART_Transmit(data>>24);
	USART_Transmit(data>>16);
	USART_Transmit(data>>8);
	USART_Transmit(data);
}



void Debug_Print_HEX_to_ASCII(unsigned char HEXin)
{
	unsigned char MSBnib;
	unsigned char LSBnib;

	/* Spilt into nibbles */
	MSBnib = (HEXin & 0xF0) >> 0x04;
	LSBnib = HEXin & 0x0F;

	/* convert MSBnib to ASCII */
	if (MSBnib <= 0x09)
		MSBnib = MSBnib + 0x30;
	else
		MSBnib = MSBnib + 0x37;


	/* convert LSBnib to ASCII */
	if (LSBnib <= 0x09)
		LSBnib = LSBnib + 0x30;
	else
		LSBnib = LSBnib + 0x37;


	/* Transmit */
	USART_Transmit(MSBnib);
	USART_Transmit(LSBnib);
}



void Debug_Print_HEX_to_ASCII_W_CR(unsigned char HEXin)
{
	Debug_Print_HEX_to_ASCII(HEXin);
	USART_Transmit(0x0D);
}



void Debug_Print(unsigned char errorNum, unsigned char error)
{
	/* four R's so easier to spot in UART dump */
	/* ERRR = 45 52 52 52 52 */
	USART_Transmit('E');
	USART_Transmit('R');
	USART_Transmit('R');
	USART_Transmit('R');
	USART_Transmit('R');
	USART_Transmit(errorNum);
	USART_Transmit(':');
	USART_Transmit('0');
	USART_Transmit('x');
	Debug_Print_HEX_to_ASCII(error);
	USART_Transmit(0x0D);

}



void Debug_Detailed(unsigned char cFile,unsigned char function,char ASCIIchar,unsigned char errorCode)
{
	/* 4 R's so easier to spot in UART dump */
	/* ERRR = 45 52 52 52 52 2C */
	USART_Transmit('E');
	USART_Transmit('R');
	USART_Transmit('R');
	USART_Transmit('R');
	USART_Transmit('R');
	USART_Transmit(',');

	if(cFile == 0) /* main.c */
	{
		USART_Transmit('m');
		USART_Transmit('a');
		USART_Transmit('i');
		USART_Transmit('n');
		USART_Transmit(',');
	}
	else if(cFile == 1) /* FTDI_IO.c */
	{
		USART_Transmit('F');
		USART_Transmit('T');
		USART_Transmit('D');
		USART_Transmit('I');
		USART_Transmit('_');
		USART_Transmit('I');
		USART_Transmit('O');
		USART_Transmit(',');

		switch(function)
		{
			case 8: /* FTDI_DRD */
			{
				USART_Transmit('D');
				USART_Transmit('R');
				USART_Transmit('D');
			}
			break;

			case 9: /* FTDI_3E0D_Check */
			{
				USART_Transmit('3');
				USART_Transmit('E');
				USART_Transmit('0');
				USART_Transmit('D');
			}
			break;

			case 10: /* FTDI_3E0D_Check */
			{
				USART_Transmit('U');
				USART_Transmit('S');
				USART_Transmit('B');
				USART_Transmit('_');
				USART_Transmit('R');
				USART_Transmit('X');
			}
			break;

			default: /* not found */
			{
				USART_Transmit('N');
				USART_Transmit('F');
			}
			break;
		}
	}
	else if(cFile == 2) /* PTP_basic.c */
	{
		USART_Transmit('P');
		USART_Transmit('T');
		USART_Transmit('P');
		USART_Transmit('_');
		USART_Transmit('b');
		USART_Transmit('a');
		USART_Transmit('s');
		USART_Transmit('i');
		USART_Transmit('c');
		USART_Transmit(',');
		
		switch(function)
		{
			case 1: /* PTP_Container_TX */
			{
				USART_Transmit('C');
				USART_Transmit('.');
				USART_Transmit('T');
				USART_Transmit('X');
			}
			break;

			case 2: /* PTP_Container_RX */
			{
				USART_Transmit('C');
				USART_Transmit('.');
				USART_Transmit('R');
				USART_Transmit('X');
			}
			break;

			case 3: /* PTP_OpenSession */
			{
				USART_Transmit('O');
				USART_Transmit('.');
				USART_Transmit('S');
				USART_Transmit('e');
				USART_Transmit('s');
				USART_Transmit('s');
			}
			break;

			case 4: /* PTP_CloseSession */
			{
				USART_Transmit('C');
				USART_Transmit('.');
				USART_Transmit('S');
				USART_Transmit('e');
				USART_Transmit('s');
				USART_Transmit('s');
			}
			break;

			case 6: /* PTP_CANON_WaitForEvent */
			{
				USART_Transmit('W');
				USART_Transmit('.');
				USART_Transmit('E');
				USART_Transmit('v');
				USART_Transmit('e');
				USART_Transmit('n');
				USART_Transmit('t');
			}
			break;

			default: /* not found */
			{
				USART_Transmit('N');
				USART_Transmit('F');
			}
			break;

		}
	}
	else
	{
		USART_Transmit('O');
		USART_Transmit('t');
		USART_Transmit('h');
		USART_Transmit('e');
		USART_Transmit('r');
		USART_Transmit(',');
	}

	USART_Transmit(',');
	USART_Transmit(ASCIIchar);
	USART_Transmit(',');
	USART_Transmit('0');
	USART_Transmit('x');
	Debug_Print_HEX_to_ASCII(errorCode);
	USART_Transmit(0x0D);

}



void Debug_Print_ASCII_String(char *inString)
{
	while(*inString)
		USART_Transmit(*inString++);	
}
